<?php

/*
  Plugin Name: EntryWizard
  Plugin URI: http:
  Description:  Uploading by logged-in users of sets of image files and associated data. Administrators may download the images together with the data in spreadsheet form.
  Version: 1.2.25
  Author: Josie Stauffer
  Author URI:
  License: GPL2
*/
/*  Copyright 2012  Josie Stauffer  (email : )

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2, as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

defined( 'ABSPATH' ) or exit;   // show a blank page if try to access this file directly

define( 'EWZ_CURRENT_VERSION', '1.2.25' );

define( 'EWZ_PLUGIN_DIR', plugin_dir_path( __FILE__ ) );
define( 'EWZ_CUSTOM_DIR', plugin_dir_path( __FILE__ ) );
define( 'EWZ_REQUIRED_WP_VERSION', '3.5' );
define( 'EWZ_REQUIRED_PHP_VERSION', '5.4.0' );

/*
 * INCLUDES
 */
require_once( EWZ_PLUGIN_DIR . 'classes/ewz-setup.php' );
require_once( EWZ_PLUGIN_DIR . 'classes/ewz-webform.php' );

if ( is_admin() ) {
    require_once( EWZ_PLUGIN_DIR . '/includes/ewz-admin.php' );
}

// this is needed for admin, too, because the ajax function runs as admin
require_once( EWZ_PLUGIN_DIR . '/includes/ewz-upload.php' );
require_once( EWZ_PLUGIN_DIR . '/includes/ewz-followup.php' );

/*
 * HOOKS
 */
register_activation_hook( __FILE__,   array( 'Ewz_Setup', 'activate_or_install_ewz' ) );
// no uninstall hook, use the uninstall.php file


/**
 * ACTIONS
 */
add_action('plugins_loaded', 'ewz_requires_version', 5 );   // first useable hook

// in the admin area we add another function after this, so make sure we know its priority
add_action( 'init', 'ewz_set_dev_env', 1 );

add_action( 'wp_enqueue_scripts', 'ewz_add_stylesheet' );

add_action( 'plugins_loaded', 'ewz_init_globals', 10 );

add_action( 'admin_init', 'ewz_check_for_db_updates', 8 );
        
// Needed for cron job to auto-close webform
add_action( 'init', 'Ewz_Webform::schedule_close' );

// delete items when a user is deleted
add_action( 'deleted_user', 'Ewz_Item::delete_user_items' );

/*
 * SHORTCODES
 */

/* Set up the EWZShortcodes menu in tinyMCE */
function ewz_shortcode_menu($screen) {
    assert( is_object( $screen ) );
    // only hook up these filters if 
    //    in the admin panel,
    //    editing a post, page or custom post type ( had issues with extra stuff the profile from other plugins ),
    //    the current user has permission to edit posts and pages, 
    //    the current user has some entrywizard permissions
    if ( current_user_can( 'edit_posts' ) && 
         current_user_can( 'edit_pages' ) && 
         Ewz_Permission::can_see_webform_page() && 
         $screen->post_type) {
        add_filter( 'mce_buttons_2', 'ewz_mce_button' );
        add_filter( 'mce_external_plugins', 'ewz_mce_plugin' );

        foreach( array('post.php','post-new.php') as $hook ){
            add_action( "admin_head-$hook", 'ewz_admin_head' );
        }     
    }
}
// need to be this late to have access to screen
add_action( 'current_screen', 'ewz_shortcode_menu', 10 ,1 );

/* pass the data to javascript */
function ewz_admin_head(){
    $ewzdata = array();
    $ewzdata['webforms'] = array_values(array_filter( Ewz_Webform::get_all_webforms( 'can_view_webform' )));
     ?>
<script type='text/javascript'>
     var EWZdata =  <?php echo json_encode( $ewzdata ); ?>;
</script>
    <?php
}  


/* pass the button data to mce_buttons filter */
function ewz_mce_button( $buttons ) {
    assert( is_array( $buttons ) );
    array_push( $buttons,'ewz_shortcodes');
    return $buttons;
}
 
/* pass the javascript file name to mce_external_plugins filter */
function ewz_mce_plugin( $plugins ) {
    assert( is_array( $plugins ) );
    $plugins['ewz_shortcodes'] = plugins_url() . '/entrywizard/javascript/ewz-shortcodes.js?ewzv=' . EWZ_CURRENT_VERSION;
    return $plugins;
}


if ( ! is_admin() ) { 
    // adding this for all admin pages triggers warnings for some themes and plugins
    // just add it specifically for the ajax calls when they are run in ewz-admin.php
    add_shortcode( 'ewz_show_webform', 'ewz_show_webform' );
    add_shortcode( 'ewz_followup', 'ewz_followup' );
}

/*
 * UPDATES
 */
function ewz_check_for_db_updates(){          
    $ewz_data_version = get_option( 'ewz_data_version' );
    if ( $ewz_data_version && version_compare( $ewz_data_version, EWZ_CURRENT_VERSION,  '<' ) ){
        // 0.9.6 added new apply_prefix column to webforms table 
        // and changed 'min_img_area' to 'min_longest_dim' in fields/fdata
        if ( version_compare( $ewz_data_version, '0.9.6', '<' ) ){
            error_log( "EWZ: updating $ewz_data_version to 0.9.6" );
            Ewz_Setup::activate_or_install_ewz();
            delete_option( 'ewz_db_version' );
            Ewz_Field::change_min_area_to_dim();    
        }
        // 0.9.8 added attach_prefs to webform table
        if ( version_compare( $ewz_data_version, '0.9.8', '<' ) ){
            error_log( "EWZ: updating $ewz_data_version to 0.9.8" );
            Ewz_Setup::activate_or_install_ewz();
        }
        // 1.0.0 added upload_date to item table
        if ( version_compare( $ewz_data_version, '1.0.0', '<' ) ){
            error_log( "EWZ: updating $ewz_data_version to 1.0.0" );
            Ewz_Setup::activate_or_install_ewz();
            Ewz_Item::set_upload_date();
        }
        // 1.1.0 added num_items to webform table and override to layout table
        if ( version_compare( $ewz_data_version, '1.1.0', '<' ) ){
            error_log( "EWZ: updating $ewz_data_version to 1.1.0" );
            Ewz_Setup::activate_or_install_ewz();
            Ewz_Webform::set_num_items();
        }
        // 1.2.0 added append to fields table
        if ( version_compare( $ewz_data_version, '1.2.0', '<' ) ){
            error_log( "EWZ: updating $ewz_data_version to 1.2.0" );
            Ewz_Setup::activate_or_install_ewz();
        }
        // 1.2.3 added gen_fname to webforms table
        if ( version_compare( $ewz_data_version, '1.2.3', '<' ) ){
            error_log( "EWZ: updating $ewz_data_version to 1.2.3" );
            Ewz_Setup::activate_or_install_ewz();
        }
        // 1.2.4 htaccess and index file in EWZ_IMG_UPLOAD_DIR
        if ( version_compare( $ewz_data_version, '1.2.4', '<' ) ){
            error_log( "EWZ: updating $ewz_data_version to 1.2.4" );
            Ewz_Setup::protect_uploads();
        }  
        // 1.2.10 added webform_order to webform table, layout_order to layout table
        if ( version_compare( $ewz_data_version, '1.2.10', '<' ) ){
            error_log( "EWZ: updating $ewz_data_version to 1.2.10" );
            Ewz_Setup::activate_or_install_ewz();
            Ewz_Webform::set_webform_order();
            Ewz_Layout::set_layout_order();
        } 
        // 1.2.18 allowed for longer prefixes
        if ( version_compare( $ewz_data_version, '1.2.18', '<' ) ){
            error_log( "EWZ: updating $ewz_data_version to 1.2.18" );
            Ewz_Setup::activate_or_install_ewz();
        }
        // 1.2.21 added "notes" column to layout table
        if ( version_compare( $ewz_data_version, '1.2.21', '<' ) ){
            error_log( "EWZ: updating $ewz_data_version to 1.2.21" );
            Ewz_Setup::activate_or_install_ewz();
        }

        update_option( 'ewz_data_version', EWZ_CURRENT_VERSION );
    }
    if( !$ewz_data_version ){
        update_option( 'ewz_data_version', EWZ_CURRENT_VERSION );
    }
}


function ewz_add_stylesheet() {
    wp_register_style( 'jquery-ui-dialog', includes_url() . "/css/jquery-ui-dialog.css" );
    wp_register_style( 'ewz-style', 
                       plugins_url( 'styles/entrywizard.css', __FILE__ ), 
                       array('jquery-ui-dialog'), 
                       EWZ_CURRENT_VERSION );
    wp_enqueue_style( 'ewz-style' );

    wp_register_script(
                       'ewz-ucommon',
                       plugins_url( 'javascript/ewz-user-common.js', __FILE__ ),
                       array( 'jquery', 'jquery-ui-core','jquery-ui-widget', 'jquery-ui-dialog', 'jquery-ui-position'),
                       EWZ_CURRENT_VERSION,
                       false
                      );
    // only register the shortcode scripts here, enqueue them within the shortcode so they are only 
    // loaded when required
    wp_register_script(
                       'ewz-upload',
                       plugins_url( 'javascript/ewz-upload.js', __FILE__ ),
                       array( 'jquery', 'ewz-ucommon' ),
                       EWZ_CURRENT_VERSION,
                       true      // in footer, so $ewzG has been defined
                      );
    wp_register_script(
                       'ewz-followup',
                       plugins_url( 'javascript/ewz-followup.js', __FILE__ ),
                       array( 'jquery', 'ewz-ucommon'  ),
                       EWZ_CURRENT_VERSION,
                       true      // in footer, so $ewzG has been defined
                      );

 }

function ewz_set_dev_env(){
    if ( is_file( plugin_dir_path( __FILE__ ). 'DEVE_ENV' )   // only true in development environment
        && ! ( isset( $_POST['action'] ) && ( 'heartbeat' == $_POST['action'] )  )
         && !preg_match( '/wp-cron.php$/', $_SERVER['REQUEST_URI'] )  
      ){   
        /*
         * ASSERT OPTIONS
         */
        assert_options( ASSERT_ACTIVE, 1 );
        assert_options( ASSERT_BAIL, false );
        assert_options( ASSERT_WARNING, false );
        function ewz_assert_handler( $file, $line, $code )
        {
            // no assert
            error_log( "EWZ: PHP Assertion Failed at $file: $line: $code" );
            ewztrace();
        }
        assert_options( ASSERT_CALLBACK, 'ewz_assert_handler' );

        
        /* 
         * LOGGING 
         */
        define( 'EWZ_DBG', true );
        $is_admin = is_admin() ? 'ADMIN' : '';
        //  stop logging of pwd
        $p = $_POST;
        unset($p['pwd']);
        if( !preg_match( '/doing_wp_cron/', $_SERVER['REQUEST_URI'] ) ){
            error_log( "EWZ: ~~~~~~~~ Starting entrywizard.php  $is_admin ~~~~~~~ \n"
                       . 'URI:' . $_SERVER['REQUEST_URI'] . "\n" 
                       . 'GET:   ' . var_export( $_GET, true )
                       . 'POST:  ' . var_export( $p, true )
                       . 'FILES: ' . var_export( $_FILES, true )
                      );
        }
        /*
         * TESTING
         */
        if( is_file( plugin_dir_path( __FILE__ ). 'GD_ENV' ) ){
            add_filter( 'wp_image_editors', 'ewz_qa_image_editor' );
        }                

    } else {
        define( 'EWZ_DBG', false );
        assert_options( ASSERT_ACTIVE, 0 );
    }
}
    
/*
 * For testing only -- allows for testing with either image editor
 */
function ewz_qa_image_editor( $editors ) {
    // no assert
    return array('WP_Image_Editor_GD', 'WP_Image_Editor_Imagick');
}



function ewz_init_globals(){
    global $wpdb;

    load_plugin_textdomain(
                            'entrywiz', false,
                            basename( dirname( __FILE__ ) ) . '/languages'
                           );

    define( 'EWZ_PREFIX', $wpdb->prefix );

    define( 'EWZ_LAYOUT_TABLE',  EWZ_PREFIX . 'ewz_layout' );
    define( 'EWZ_FIELD_TABLE',   EWZ_PREFIX . 'ewz_field' );
    define( 'EWZ_WEBFORM_TABLE', EWZ_PREFIX . 'ewz_webform' );
    define( 'EWZ_ITEM_TABLE',    EWZ_PREFIX . 'ewz_item' );

    $uploads_dir = wp_upload_dir();
    define( 'EWZ_IMG_UPLOAD_DIR', $uploads_dir['basedir'] . '/ewz_img_uploads' );
    define( 'EWZ_IMG_UPLOAD_URL', $uploads_dir['baseurl'] . '/ewz_img_uploads' );
    define( 'EWZ_IMG_UPLOAD_SUBDIR', 'ewz_img_uploads' );


    define( 'EWZ_DEFAULT_DIM',  1280 );           // default max image dimension in pixels
    define( 'EWZ_DEFAULT_MIN_LONGEST',  100 );    // default minimum longest image dimension in pixels
    define( 'EWZ_MAX_STRING_LEN', 50 );           // default max length of text input field
    define( 'EWZ_MAX_FIELD_WIDTH', 10 );          // default max field width of text input field

    define( 'EWZ_FILE_DOWNLOAD_TIME', 30 );       // number of seconds to add to max time allowance for every 25 files
    define( 'UPLOAD_ERR_EMPTY',   5 );            // error if uploaded file is empty -- see includes/ewz-upload.php


    $maxnumitems = ini_get( 'max_file_uploads' );
    if ( ! $maxnumitems ){
        $maxnumitems = 50;
    }
    $maxfilesize = ini_get( 'upload_max_filesize' );
    if ( ! $maxfilesize ){
        $maxfilesize = '5M';
    }
    $postmaxsize = ini_get( 'post_max_size' );
    if ( ! $postmaxsize ){
        $postmaxsize = '100M';
    }


    define( 'EWZ_MAX_NUMITEMS', (int) $maxnumitems );
    define( 'EWZ_MAX_TOTAL_MB', ewz_to_mb( $postmaxsize ) );      // default max total post size in MB
    define( 'EWZ_MAX_SIZE_MB',  ewz_to_mb( $maxfilesize ) );      // default max upload image size in MB
    define( 'EWZ_MAX_SIZE_BYTES', ewz_to_bytes( $maxfilesize ) ); // default max upload image size in bytes
}


/**
 * Transform a string like 100G or 50K to megabytes
 *
 * Works for an integer followed by 'm', 'mb', 'g','gb', 'k', 'kb'
 * or the same in upper case
 *
 * @param   $sizestring
 * @return  integer
 */
function ewz_to_mb( $sizestring )
{
    assert( is_string( $sizestring ) );
    assert( preg_match( '/^(\d+)[mMgGkK][bB]?\s*$/', $sizestring ) );

    $val = strtolower( trim( $sizestring ) );
    $mat = array();
    preg_match( '/^(\d+)(\D+)/', $val, $mat );
    assert( 3 == count( $mat ) );
    $num  = $mat[1];
    $code = $mat[2];

    if ( preg_match( '/mb?/', $code ) ) {
        return ( int )$num;
    }
    if ( preg_match( '/gb?/', $code ) ) {
        return ( int )($num * 1024);
    }
    if ( preg_match( '/kb?/', $code ) ) {
        return ( int )($num / 1024);
    }
    return 0;
}


/**
 * Transform a string like 100G or 50K to bytes
 *
 * Works for an integer followed by 'm', 'mb', 'g','gb', 'k', 'kb'
 * or the same in upper case
 *
 * @param   $sizestring
 * @return  integer
 */
function ewz_to_bytes( $sizestring )
{
    assert( is_string( $sizestring ) );
    assert( preg_match( '/^(\d+)[mMgGkK][bB]?\s*$/', $sizestring ) );

    $val = strtolower( trim( $sizestring ) );
    $mat = array();
    preg_match( '/^(\d+)(\D+)/', $val, $mat );
    assert( 3 == count( $mat ) );
    $num  = $mat[1];
    $code = $mat[2];

    if ( preg_match( '/mb?/', $code ) ) {
        return $num * 1024;
    }
    if ( preg_match( '/gb?/', $code ) ) {
        return $num * 1048576;
    }
    if ( preg_match( '/kb?/', $code ) ) {
        return $num;
    }
    return 0;
}

/**
 *
 * Warn if required versions of WP and PHP not available
 *
 */
function ewz_requires_version(){
   $reqvers = ewz_required_versions_warning();
   if ( $reqvers ) {
       add_action( 'admin_notices', 'ewz_admin_notice' );
   }  
}
   
/*
 * Return a string like "WordPress version XXX and PHP version yyy"
 * if the required versions of WP and PHP are not available.
 * Also log to error log.
 */
function ewz_required_versions_warning(){
   global $wp_version;
   $reqvers = '';

   // version_compare returns -1 if left is lower than right, 0 if they are equal, 1 if right is lower than left
   if ( version_compare( EWZ_REQUIRED_WP_VERSION, $wp_version ) == 1 ) {
      $reqvers = 'WordPress version ' . EWZ_REQUIRED_WP_VERSION;
      error_log("EWZ: wordpress version not supported");
   }
   if ( version_compare( EWZ_REQUIRED_PHP_VERSION, PHP_VERSION ) == 1 ) {
       error_log("EWZ: php version not supported");
      if( $reqvers ){
            $reqvers .= " and ";
      }
      $reqvers .= 'PHP version ' . EWZ_REQUIRED_PHP_VERSION;
   }
   return $reqvers;
}

function ewz_admin_notice() {
    $msg = ewz_required_versions_warning();
    if( $msg ){
        print "<div class='error'><h2>EntryWizard Warning</h2> EntryWizard requires at least <strong>$msg </strong></div>";
    }
}

/* * **************************************
 * Utility to output data structures in debug.log
 *
 * For debugging purposes
 * @param string $str descriptive string
 * @param mixed $obj object to be dumped
 * @return void
 * */

function ewztrace( $ignore = 2 )
{
    assert( is_int( $ignore ) );

    if ( version_compare( phpversion(),  '5.3.6' ) >= 0 ){
        foreach ( debug_backtrace( DEBUG_BACKTRACE_IGNORE_ARGS ) as $k => $vv ) { 
            if ($k < $ignore) { 
                continue; 
            } 
            if( isset( $vv['file'] ) ){
                if( strpos( 'entrywizard', $vv['file'] ) >= 0  || strpos( 'ewz', $vv['file'] >= 0 ) ){
                    $shrtfile = str_replace( '/home/josie/www/rhcc_site/wp-content/plugins/', '', $vv['file'] );
                    error_log( 'EWZ:#' . ($k - $ignore) . ' ' . $shrtfile. '(' . $vv['line'] . '): ' . 
                               (isset($vv['class']) ? $vv['class'] . '->' : '') . $vv['function']  ); 
                }
            }
        }
    }
} 

function ewzdbg( $str, $in_obj )
{
    // no assert
    if ( defined( 'EWZ_DBG' ) && EWZ_DBG ){
        // $trace = false;
        //  if( $trace ){
        //   ewztrace();
        //}
        // no assert
        ob_start();
        var_dump( $in_obj );
        $dump = preg_replace('/=>\s*\n/', ' => ', ob_get_contents());
        ob_end_clean();

        if ( isset($in_obj )) {
            error_log(  "EWZ_DBG object $str: " . $dump );
            return;
        } else {
            error_log( 'EWZ_DBG variable: ' .  $dump );
            return;
        }
    }
}


